/*
  Copyright (C) 2010 Julien Jorge

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: julien.jorge@gamned.org
*/
#include <fstream>
#include <iostream>

/**
 * \brief Ajoute des espaces dans un flux.
 * \param os Le flux dans lequel crire.
 * \param n Le nombre d'espaces  ajouter.
 */
void do_indent( std::ostream& os, unsigned int n )
{
  for (unsigned int i=0; i!=n; ++i)
    os << ' ';
}

/**
 * \brief Ce programme gnre un programme c++ rsolvant un problme de sac 
 *        dos lors de sa compilation.
 * \param Le nom du fichier de l'instance du problme.
 * 
 * Le rsultat est un fichier nomm argv[1]-rec.cpp prt  tre compil.
 */
int main( int argc, char* argv[] )
{
  if ( argc != 2 )
    {
      std::cout << argv[0] << " instance" << std::endl;
      return 1;
    }

  std::ifstream in_f(argv[1]);
  std::string out_name(argv[1]);
  out_name += "-rec.cpp";

  std::ofstream out_f( out_name.c_str() );

  unsigned int c; // la capacit du sac
  in_f >> c;
  unsigned int indent(0);

  out_f << "#include \"kp-rec.hpp\"\n"
        << "#include <iostream>\n\n"
        << "int main()\n"
        << "{\n";
  indent += 2;

  do_indent(out_f, indent);
  out_f << "typedef\n";
  indent +=2;

  unsigned int p, w; // profit et poids
  while (in_f >> p >> w)
    {
      do_indent(out_f, indent);
      out_f << "type_list< item<" << p << ", " << w << ">,\n";
      indent +=2;
    }

  do_indent(out_f, indent);
  out_f << "empty_list\n";
  indent-=2;

  while ( indent!=2 )
    {
      do_indent(out_f, indent);
      out_f << ">\n";
      indent-=2;
    }

  do_indent(out_f, indent);
  out_f << " kp;\n\n";

  do_indent(out_f, indent);
  out_f << "std::cout << \"result=\" << solve_kp<" << c
        << ", kp>::result << '\\n'\n";
  out_f << "          << std::endl;\n\n";

  do_indent(out_f, indent);
  out_f << "return 0;\n}" << std::endl;

  return 0;
}
