#ifndef __KP_REC_HPP__
#define __KP_REC_HPP__

#include "kp.hpp"

/**
 * \brief Une classe qui rsout un problme de sac  dos en utilisant les
 *        quations de Bellman. La valeur optimale du problme est stocke dans
 *        une constante membre statique nomme result.
 * \param RemainingCapacity La capacit restante dans le sac.
 * \param ItemList Les objets sur lesquels nous faisons une slection.
 */
template<unsigned int RemainingCapacity, typename ItemList>
struct solve_kp;

/**
 * \brief Spcialisation pour le cas d'une liste vide d'objets. Quelle que soit
 *        la capacit, la valeur de la solution est zro.
 */
template<unsigned int RemainingCapacity>
struct solve_kp<RemainingCapacity, empty_list>
{
  static const unsigned int result = 0;
};

/* Dclaration complte. */
template<unsigned int RemainingCapacity, typename ItemList>
struct solve_kp
{
  typedef typename ItemList::head head;
  typedef typename ItemList::tail tail;

  static const unsigned int result =
    (head::weight > RemainingCapacity) ?
      solve_kp<RemainingCapacity, tail>::result :
      max_value
      <
        solve_kp<RemainingCapacity, tail>::result,
        solve_kp<RemainingCapacity - head::weight, tail>::result + head::profit
      >::result;
}; // struct solve_kp

#endif // __KP_REC_HPP__
